==> eg_particle_interactions-v0.8.1-fabric-mc1.21.8-sources.jar: games/enchanted/eg_particle_interactions/common/particle/swirling/Ember.java <==
package games.enchanted.eg_particle_interactions.common.particle.swirling;

import games.enchanted.eg_particle_interactions.common.duck.ParticleAccess;
import games.enchanted.eg_particle_interactions.common.util.MathHelpers;
import net.minecraft.class_2400;
import net.minecraft.class_3532;
import net.minecraft.class_3999;
import net.minecraft.class_4002;
import net.minecraft.class_638;
import net.minecraft.class_703;
import net.minecraft.class_707;
import net.minecraft.class_765;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public class Ember extends SwirlingParticle {
    protected Ember(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 spriteSet, boolean shouldSwirl) {
        super(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, shouldSwirl);
        this.setInitialVelocity(xSpeed, ySpeed, zSpeed, 0.015f);

        this.field_3844 = MathHelpers.randomBetween(-0.03f, -0.08f);

        this.field_3847 = MathHelpers.randomBetween(20, 100);

        this.rotSpeed = 0f;
        this.spinAcceleration = 0f;
        this.swirlStrength = MathHelpers.randomBetween(0.7f, 0.9f) * (level.field_9229.method_43056() ? -1 : 1);
        this.swirlPeriod = MathHelpers.randomBetween(708, 720);

        this.field_17867 = 2.5f/32f;
        this.method_3080(1/32f, 1/32f);

        ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(true);
    }

    @Override
    public void applyGravity() {
        this.field_3869 = -this.field_3844;
    }

    @Override
    protected void renderTick(float partialTicks) {
        float percentageAge = (float) this.field_3866 / this.field_3847;
        if(percentageAge > 0.8) {
            float finalA = 1 - (((class_3532.method_16439(partialTicks, this.field_3866, this.field_3866 + 1f) / this.field_3847) - 0.8f) * 5f);
            if(finalA < 0) {
                this.field_3841 = 0f;
                return;
            }
            this.field_3841 = finalA;
        }
    }

    @Override
    public int method_3068(float partialTicks) {
        return class_765.field_32767;
    }

    @Override
    public @NotNull class_3999 method_18122() {
        return class_3999.field_17829;
    }

    public static class EmberProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public EmberProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Nullable
        @Override
        public class_703 createParticle(@NotNull class_2400 type, @NotNull class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed) {
            return new Ember(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, true);
        }
    }
}

==> eg_particle_interactions-v0.8.4-fabric-mc1.21.10-sources.jar: games/enchanted/eg_particle_interactions/common/particle/swirling/Ember.java <==
package games.enchanted.eg_particle_interactions.common.particle.swirling;

import games.enchanted.eg_particle_interactions.common.duck.ParticleAccess;
import games.enchanted.eg_particle_interactions.common.util.MathHelpers;
import net.minecraft.class_2400;
import net.minecraft.class_3532;
import net.minecraft.class_4002;
import net.minecraft.class_5819;
import net.minecraft.class_638;
import net.minecraft.class_703;
import net.minecraft.class_707;
import net.minecraft.class_765;
import org.jetbrains.annotations.Nullable;

public class Ember extends SwirlingParticle {
    protected Ember(class_638 level, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, class_4002 spriteSet, boolean shouldSwirl) {
        super(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, shouldSwirl);
        this.setInitialVelocity(xSpeed, ySpeed, zSpeed, 0.015f);

        this.field_3844 = MathHelpers.randomBetween(-0.03f, -0.08f);

        this.field_3847 = MathHelpers.randomBetween(20, 100);

        this.rotSpeed = 0f;
        this.spinAcceleration = 0f;
        this.swirlStrength = MathHelpers.randomBetween(0.7f, 0.9f) * (level.field_9229.method_43056() ? -1 : 1);
        this.swirlPeriod = MathHelpers.randomBetween(708, 720);

        this.setScale(2.5f/32f);
        this.method_3080(1/32f, 1/32f);

        ((ParticleAccess) this).eg_particle_interactions$setBypassMovementCollisionCheck(true);
    }

    @Override
    public void applyGravity() {
        this.field_3869 = -this.field_3844;
    }

    @Override
    protected void renderTick(float partialTicks) {
        float percentageAge = (float) this.field_3866 / this.field_3847;
        if(percentageAge > 0.8) {
            float finalA = 1 - (((class_3532.method_16439(partialTicks, this.field_3866, this.field_3866 + 1f) / this.field_3847) - 0.8f) * 5f);
            if(finalA < 0) {
                this.alpha = 0f;
                return;
            }
            this.alpha = finalA;
        }
    }

    @Override
    public int method_3068(float partialTicks) {
        return class_765.field_32767;
    }

    @Override
    protected ParticleLayer getParticleLayer() {
        return ParticleLayer.TRANSLUCENT;
    }

    public static class EmberProvider implements class_707<class_2400> {
        private final class_4002 spriteSet;

        public EmberProvider(class_4002 spriteSet) {
            this.spriteSet = spriteSet;
        }

        @Override
        public @Nullable class_703 createParticle(
            class_2400 options,
            class_638 level,
            double x,
            double y,
            double z,
            double xSpeed,
            double ySpeed,
            double zSpeed
            //? if minecraft: > 1.21.8 {
            , class_5819 random
            //?}
        ) {
            return new Ember(level, x, y, z, xSpeed, ySpeed, zSpeed, spriteSet, true);
        }
    }
}
